;;
;; qr-payment.scm
;;
;; QR payment generator.
;;
;; ISC License
;;
;; Copyright 2023-2025 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit qr-payment))

(module
    qr-payment
    (
     make-qrp
     make-brmlab-qrp
     )

  (import scheme
	  (chicken format)
	  (chicken string)
	  (chicken base))

  (define (make-empty-qrp . vs)
    (let ((v (if (null? vs) "1.0" (car vs))))
      (list v "SPD")))

  (define (add-field-to-qrp qrp key value)
    (cons (format "~A:~A" key value)
	  qrp))

  (define (serialize-qrp qrp)
    (string-intersperse (reverse qrp) "*"))

  (define (ensure-amount-format amt)
    (if (string? amt)
	amt
	amt))

  (define (make-qrp iban amt cc vs msg)
    (let loop ((keys '(ACC AM CC MSG X-CS))
	       (vals (list iban (ensure-amount-format amt) cc msg vs))
	       (qrp (make-empty-qrp)))
      (if (null? keys)
	  (serialize-qrp qrp)
	  (loop (cdr keys)
		(cdr vals)
		(add-field-to-qrp qrp (car keys) (car vals))))))

  (define (make-brmlab-qrp amt cc vs)
    (let ((iban (if (equal? cc "CZK")
		    "CZ0520100000002500079551"
		    (if (equal? cc "EUR")
			"CZ9320100000002100079552"
			(error "Invalid currency!")))))
      (make-qrp iban amt cc vs "Brmlab")))

  (print (make-brmlab-qrp 1000 "CZK" 1234))

  )
